@@ -211,6 +211,7 @@ static TranslationEntry translations [] = {
         { "org.gnome.desktop.interface", "cursor-size",            "Gtk/CursorThemeSize",     translate_int_int },
         { "org.gnome.desktop.interface", "show-input-method-menu", "Gtk/ShowInputMethodMenu", translate_bool_int },
         { "org.gnome.desktop.interface", "show-unicode-menu",      "Gtk/ShowUnicodeMenu",     translate_bool_int },
+        { "org.gnome.desktop.interface", "automatic-mnemonics",    "Gtk/AutoMnemonics",       translate_bool_int },
 
         { "org.gnome.desktop.sound", "theme-name",                 "Net/SoundThemeName",            translate_string_string },
         { "org.gnome.desktop.sound", "event-sounds",               "Net/EnableEventSounds" ,        translate_bool_int },
@@ -612,7 +613,7 @@ terminate_cb (void *data)
         }
 
         *terminated = TRUE;
-
+        g_warning ("X Settings Manager is terminating");
         gtk_main_quit ();
 }
 
@@ -714,6 +715,7 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
         manager->priv->gtk = gsd_xsettings_gtk_new ();
         g_signal_connect (G_OBJECT (manager->priv->gtk), "notify::gtk-modules",
                           G_CALLBACK (gtk_modules_callback), manager);
+        gtk_modules_callback (manager->priv->gtk, NULL, manager);
 
         /* Xft settings */
         update_xft_settings (manager);
@@ -751,14 +753,22 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
                 p->managers = NULL;
         }
 
-        g_object_unref (manager->priv->plugin_settings);
+        if (p->plugin_settings != NULL) {
+                g_object_unref (p->plugin_settings);
+                p->plugin_settings = NULL;
+        }
+
         stop_fontconfig_monitor (manager);
 
-        g_hash_table_destroy (p->settings);
-        p->settings = NULL;
+        if (p->settings != NULL) {
+                g_hash_table_destroy (p->settings);
+                p->settings = NULL;
+        }
 
-        g_object_unref (p->gtk);
-        p->gtk = NULL;
+        if (p->gtk != NULL) {
+                g_object_unref (p->gtk);
+                p->gtk = NULL;
+        }
 }
 
 static void
